-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI board for CNC control/whatever.   --
--                                                               --
-- It has 32 optoisolated parallel inputs, 32 open drain outputs --
-- with 5V power for external optocouplers, 10 galvanically      --
-- isolated differential transmitters / 2 receivers (for making  --
-- several SPI interfaces with up to 1 MHz speed or whatever,)   --
-- and 42 unprogrammed I/O lines for extension boards. It is     --
-- supposed to work with CENTIPEDE-BRK breakout board that has   --
-- screw terminals for all signals, 4 signal and 4 power relays, --
-- 5V/3A universal input power supply for feeding secondary side --
-- (i.e. CNC machine side) sensors and whatever else, 2-channel  --
-- SPI DAC (TLV5617A) with 0-10V outputs, and 4-channel SPI ADC  --
-- (TLV1544) with 0-10V inputs.                                  --
--                                                               --
-- More info available from www.ksilabs.com.                     --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

---------------
-- LIBRARIES --
---------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;
use work.mach3_components.all;

------------
-- ENTITY --
------------

entity centipede_pci is
generic (
	MACH_FIFO_DEPTH		: natural := 20;
	ZEROS_22		: std_logic_vector(21 downto 0) := (others => '0');
	DOUT_INIT_VAL		: std_logic_vector(31 downto 0) := x"00000000";
	DOUT_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0000";
	DIN_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0001";
	DAC_A_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0010";
	DAC_B_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0011";
	ADC_0_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0100";
	ADC_1_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0101";
	ADC_2_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0110";
	ADC_3_REG_ADR		: std_logic_vector(6 downto 0) := b"000_0111";
	IRQSTAT_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1000";
	IRQMASK_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1001";
	IRQSTAT_MASKED_ADR	: std_logic_vector(6 downto 0) := b"000_1010";
	IDX_0_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1011";
	IDX_1_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1100";
	IDX_2_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1101";
	IDX_3_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1110";
	IDX_4_REG_ADR		: std_logic_vector(6 downto 0) := b"000_1111";
	IDX_5_REG_ADR		: std_logic_vector(6 downto 0) := b"001_0000";
	MACH3CFG_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0001";
	MACH3CFG2_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0010";
	MACH3CTL_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0011";
	MACH3STAT_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0100";
	MACH3FIFO_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0101";
	MACH3ENC_REG_ADR	: std_logic_vector(6 downto 0) := b"001_0110";
	EX_0_31_REG_ADR		: std_logic_vector(6 downto 0) := b"001_0111";
	EX_32_41_REG_ADR	: std_logic_vector(6 downto 0) := b"001_1000";
	EX_0_31_DIR_REG_ADR	: std_logic_vector(6 downto 0) := b"001_1001";
	EX_32_41_DIR_REG_ADR	: std_logic_vector(6 downto 0) := b"001_1010"
);
port (
	------------------------
	-- PCI Target 32 bits --
	------------------------

	l_clk		: in std_logic;
	l_rst		: in std_logic;
	l_ad		: inout std_logic_vector(31 downto 0);
	l_cbe		: in std_logic_vector(3 downto 0);
	l_par		: out std_logic;
	l_frame		: in std_logic;
	l_irdy		: in std_logic;
	l_trdy		: out std_logic;
	l_devsel	: out std_logic;
	l_stop		: out std_logic;
	l_idsel		: in std_logic;
	l_perr		: out std_logic;
	l_serr		: out std_logic;
	l_inta		: out std_logic;

	---------
	-- I/O --
	---------

	d_in		: in std_logic_vector(31 downto 0);
	d_out		: out std_logic_vector(31 downto 0);

	---------------
	-- Extension --
	---------------

	ext		: inout std_logic_vector(41 downto 0);

	-- HINT: Unused SPI pins can be used for making another SPI :)

	----------
	-- SPI1 --
	----------
		
	liso1		: in std_logic;
	losi1		: out std_logic;
	lpck1		: out std_logic;
	lss11		: out std_logic;
	lss12		: out std_logic;
	lss13		: out std_logic;

	----------
	-- SPI2 --
	----------
		
	liso2		: in std_logic;
	losi2		: out std_logic;
	lpck2		: out std_logic;
	lss21		: out std_logic;
	lss22		: out std_logic;
	lss23		: out std_logic;

	----------
	-- Misc --
	----------

	pci_en		: out std_logic
);
end centipede_pci;

------------------
-- ARCHITECTURE --
------------------

architecture guts of centipede_pci is

	----------------
	-- COMPONENTS --
	----------------

	---------------
	-- CONSTANTS --
	---------------
	
	-------------
	-- SIGNALS --
	-------------

	signal clk		: std_logic;
	signal nrst		: std_logic;
	signal lb_adr		: std_logic_vector(11 downto 0);
	signal lb_din		: std_logic_vector(31 downto 0);
	signal lb_dout		: std_logic_vector(31 downto 0);
	signal lb_nbe		: std_logic_vector(3 downto 0);
	signal lb_wr		: std_logic;
	signal lb_rd		: std_logic;
	signal lb_nint		: std_logic;

	signal spi_clk		: std_logic;

	signal ioaddr		: std_logic_vector(6 downto 0);
	signal irqstat_wr	: std_logic;
	signal irq		: std_logic;

	signal d_out_reg	: std_logic_vector(31 downto 0);
	signal d_in_reg		: std_logic_vector(31 downto 0);
	signal irqstat_reg	: std_logic_vector(31 downto 0);	
	signal irqmask_reg	: std_logic_vector(31 downto 0);	
	signal dac_a_reg	: std_logic_vector(9 downto 0);	
	signal dac_b_reg	: std_logic_vector(9 downto 0);	
	signal adc_0_reg	: std_logic_vector(9 downto 0);	
	signal adc_1_reg	: std_logic_vector(9 downto 0);	
	signal adc_2_reg	: std_logic_vector(9 downto 0);	
	signal adc_3_reg	: std_logic_vector(9 downto 0);	

	signal ex_data		: std_logic_vector(41 downto 0);
	signal ex_dir		: std_logic_vector(41 downto 0);

	signal dac_written	: std_logic_vector(1 downto 0);
	signal dac_done		: std_logic_vector(1 downto 0);
	signal dac_done_prev	: std_logic_vector(1 downto 0);
	signal dac_done_up	: std_logic_vector(1 downto 0);

	signal adc_read		: std_logic_vector(3 downto 0);
	signal adc_done		: std_logic_vector(3 downto 0);	
	signal adc_done_prev	: std_logic_vector(3 downto 0);	
	signal adc_done_up	: std_logic_vector(3 downto 0);

	signal spi_connected	: std_logic;
	signal spi_conn_prev	: std_logic;
	signal spi_broke	: std_logic;
	
	signal pulser_bus	: std_logic_vector(11 downto 0);
	signal axes_active	: std_logic_vector(5 downto 0);
	signal mach_irq_home	: std_logic_vector(5 downto 0);
	signal mach_irq_probe	: std_logic;
	signal mach_irq_cycle	: std_logic;
	signal mach_irq_done	: std_logic;
	signal mach_irq_fault	: std_logic;
	signal mach_irq_fovf	: std_logic;
	signal mach_fovf_clr	: std_logic;
	signal mach_enc_rd	: std_logic;
	signal mach_idx		: index_regs;
	signal mach_cfg_reg	: std_logic_vector(31 downto 0);
	signal mach_cfg2_reg	: std_logic_vector(7 downto 0);
	signal mach_ctl_reg	: std_logic_vector(31 downto 0);
	signal mach_stat_reg	: std_logic_vector(31 downto 0);
	signal mach_enc_reg	: std_logic_vector(31 downto 0);
	signal mach_enc_data	: std_logic_vector(7 downto 0);
begin

	--=====================--
	-- COMPONENT INSTANCES --
	--=====================--

	-----------------------------
	-- PCI32TTINY : PCI Target --
	-----------------------------

	u1: component pci32ttiny
	generic map (
		vendorID	=> x"feff",
		deviceID	=> x"0001",
		revisionID	=> x"00",
		subsystemID	=> x"0202",
		subsystemvID	=> x"1172",
		classcodeID	=> x"078000"
	)
	port map (
		-- General 
		pci_clk => l_clk,
		pci_rst => l_rst,
    
		-- PCI target 32bits
		pci_ad => l_ad,
		pci_cbe => l_cbe,
		pci_par => l_par,
		pci_frame => l_frame,
		pci_irdy => l_irdy,
		pci_trdy => l_trdy,
		pci_devsel => l_devsel,
		pci_stop => l_stop,
		pci_idsel => l_idsel,
		pci_perr => l_perr,
		pci_serr => l_serr,
		pci_inta => l_inta,
      
		-- Local bus
		lb_adr => lb_adr,
		lb_data_in => lb_din,
		lb_data_out => lb_dout,
		lb_nbe => lb_nbe,
		lb_nint => lb_nint,
		lb_rd => lb_rd,
		lb_wr => lb_wr
	);

	--------------------------
	-- TLV5617A DAC on SPI1 --
	--------------------------

	u2: component tlv5617a_dac
	port map (
		spi_ok => spi_connected,
		dac_a_sent => dac_done(0),
		dac_b_sent => dac_done(1),
		dac_a_data => dac_a_reg,
		dac_b_data => dac_b_reg,
		ss_pin => lss11,
		sclk_pin => lpck1,
		mosi_pin => losi1,
		clk => spi_clk,
		nrst => nrst
	);

	-------------------------
	-- TLV1544 ADC on SPI2 --
	-------------------------

	u3: component tlv1544_adc
	port map (
		spi_ok => spi_connected,
		ch0_done => adc_done(0),
		ch1_done => adc_done(1),
		ch2_done => adc_done(2),
		ch3_done => adc_done(3),
		ch0_data => adc_0_reg,
		ch1_data => adc_1_reg,
		ch2_data => adc_2_reg,
		ch3_data => adc_3_reg,
		ss_pin => lss21,
		sclk_pin => lpck2,
		mosi_pin => losi2,
		miso_pin => liso2,
		clk => spi_clk,
		nrst => nrst
	);

	-------------------------------------
	-- Mach 3 Pulsing Engine and stuff --
	-------------------------------------

	u4: component mach3_main
	generic map (
		fifo_depth => MACH_FIFO_DEPTH,
		idx_adr => (IDX_0_REG_ADR, IDX_1_REG_ADR, IDX_2_REG_ADR,
			IDX_3_REG_ADR, IDX_4_REG_ADR, IDX_5_REG_ADR),
		cfg_adr => MACH3CFG_REG_ADR,
		cfg2_adr => MACH3CFG2_REG_ADR,
		ctl_adr => MACH3CTL_REG_ADR,
		stat_adr => MACH3STAT_REG_ADR,
		fifo_adr => MACH3FIFO_REG_ADR
	)
	port map (
		pulser_bus => pulser_bus,
		axes_active => axes_active,
		irq_home => mach_irq_home,
		irq_probe => mach_irq_probe,
		irq_cycle => mach_irq_cycle,
		irq_done => mach_irq_done,
		irq_fault => mach_irq_fault,
		irq_fovf => mach_irq_fovf,
		idx => mach_idx,
		cfg_reg => mach_cfg_reg,
		cfg2_reg => mach_cfg2_reg,
		ctl_reg => mach_ctl_reg,
		stat_reg => mach_stat_reg,
		enc_reg => mach_enc_reg,
		enc_rd => mach_enc_rd,
		fovf_clr => mach_fovf_clr,
		enc_data => mach_enc_data,
		data_in	=> lb_din,
		din_reg	=> d_in_reg,
		ioaddr => ioaddr,
		rd => lb_rd,
		wr => lb_wr,
		clk => clk,
		nrst => nrst
	);

	---------------------------
	-- GPIO on Extension Bus --
	---------------------------

	u5: component extbus_gpio
	generic map (
		dreg_low_adr => EX_0_31_REG_ADR,
		dreg_hi_adr => EX_32_41_REG_ADR,
		ddir_low_adr => EX_0_31_DIR_REG_ADR,
		ddir_hi_adr => EX_32_41_DIR_REG_ADR
	)
	port map (
		ext_bus => ext,
		data_in => lb_din,
		data_out => ex_data,
		dir_out => ex_dir,
		ioaddr => ioaddr,
		rd => lb_rd,
		wr => lb_wr,
		clk => clk,
		nrst => nrst
	);

	----------
	-- MAIN --
	----------

	--------------------------------------
	-- ~516KHz clock for SPI interfaces --
	--------------------------------------
	spi_clock : process (nrst, clk)
		variable spiclk_cnt	: unsigned(5 downto 0);
	begin
		if (nrst='0') then
			spiclk_cnt := (others => '0');
		elsif (rising_edge(clk)) then
			spi_clk <= spiclk_cnt(spiclk_cnt'LEFT);
			spiclk_cnt := spiclk_cnt + 1;
		end if;
	end process;


	-----------------------------------------
	-- Latch input data into d_in register --
	-----------------------------------------
	get_d_in : process (clk)
	begin
		if (rising_edge(clk)) then
			d_in_reg <= not d_in;
		end if;
	end process;


	--------------------------------------------------------
	-- Registers read/write from LB/PCI. Only 32 bit size --
	-- access is allowed.                                 --
	--------------------------------------------------------
	reg_rw : process (nrst, clk)
	begin
		if (nrst='0') then
			d_out_reg <= DOUT_INIT_VAL;
			dac_a_reg <= (others => '0');
			dac_b_reg <= (others => '0');
			dac_written <= (others => '0');
			adc_read <= (others => '0');
		elsif (rising_edge(clk)) then
			dac_written <= (others => '0');
			adc_read <= (others => '0');
			case ioaddr is
				when DOUT_REG_ADR =>
					if (lb_wr='1') then
						d_out_reg <= lb_din;
					elsif (lb_rd='1') then
						lb_dout <= d_out_reg;
					end if;
				when DIN_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= d_in_reg;
					end if;
				when DAC_A_REG_ADR =>
					if (lb_wr='1') then
						dac_a_reg <= lb_din(9 downto 0);
						dac_written(0) <= '1';
					elsif (lb_rd='1') then
						lb_dout <= ZEROS_22 & dac_a_reg;
					end if;
				when DAC_B_REG_ADR =>
					if (lb_wr='1') then
						dac_b_reg <= lb_din(9 downto 0);
						dac_written(1) <= '1';
					elsif (lb_rd='1') then
						lb_dout <= ZEROS_22 & dac_b_reg;
					end if;
				when ADC_0_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & adc_0_reg;
						adc_read(0) <= '1';
					end if;
				when ADC_1_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & adc_1_reg;
						adc_read(1) <= '1';
					end if;
				when ADC_2_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & adc_2_reg;
						adc_read(2) <= '1';
					end if;
				when ADC_3_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & adc_3_reg;
						adc_read(3) <= '1';
					end if;
				when IRQMASK_REG_ADR =>
					if (lb_wr='1') then
						irqmask_reg <= lb_din;
					elsif (lb_rd='1') then
						lb_dout <= irqmask_reg;
					end if;
				when IRQSTAT_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= irqstat_reg;
					end if;
				when IRQSTAT_MASKED_ADR =>
					if (lb_rd='1') then
						lb_dout <= irqstat_reg and irqmask_reg;
					end if;
				when IDX_0_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(0);
					end if;
				when IDX_1_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(1);
					end if;
				when IDX_2_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(2);
					end if;
				when IDX_3_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(3);
					end if;
				when IDX_4_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(4);
					end if;
				when IDX_5_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_idx(5);
					end if;
				when MACH3CFG_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_cfg_reg;
					end if;
				when MACH3CFG2_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= x"00_00_00" & mach_cfg2_reg;
					end if;
				when MACH3CTL_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_ctl_reg;
					end if;
				when MACH3STAT_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_stat_reg;
					end if;
				when MACH3ENC_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= mach_enc_reg;
					end if;
				when EX_0_31_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ex_data(31 downto 0);
					end if;
				when EX_32_41_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & ex_data(41 downto 32);
					end if;
				when EX_0_31_DIR_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ex_dir(31 downto 0);
					end if;
				when EX_32_41_DIR_REG_ADR =>
					if (lb_rd='1') then
						lb_dout <= ZEROS_22 & ex_dir(41 downto 32);
					end if;
				when others =>
					null;
			end case;
		end if;
	end process;


	----------------------------------------------------
	-- Detect rising edges for adc_done and dac_done. --
	-- Output signals go high for 1 clock only.       --
	----------------------------------------------------
	adc_dac_prev_state : process (nrst, clk)
	begin
		if (nrst='0') then
			dac_done_prev <= (others => '0');
			adc_done_prev <= (others => '0');
		elsif (rising_edge(clk)) then
			dac_done_prev <= dac_done;
			adc_done_prev <= adc_done;
		end if;
	end process;

	dac_done_up <= dac_done and not dac_done_prev;
	adc_done_up <= adc_done and not adc_done_prev;


	--------------------------------------------------------------------
	-- Detect SPI disconnect (BRK board is either disconnected or     --
	-- powered down.) Output signal goes high for 1 clock only. It is --
	-- BRAKE detection, NOT state (i.e. it's only raised if BRK board --
	-- was connected before; no signal if it was never connected.)    --
	--------------------------------------------------------------------
	spi_conn_prev_state : process (nrst, clk)
	begin
		if (nrst='0') then
			spi_conn_prev <= '0';
		elsif (rising_edge(clk)) then
			spi_conn_prev <= spi_connected;
		end if;
	end process;

	spi_broke <= spi_conn_prev and not spi_connected;


	---------------------------------------------------------------
	-- Processing for irqstat_reg. IRQs for new ADC data arrived --
	-- since last read/clear IRQ, DAC data has been transferred  --
	-- to DACs, connection to BRK board has been lost, Mach 3    --
	-- cycle started or command complete or fault detected (IRQ  --
	-- for Windoze driver.)                                      --
	--                                                           --
	-- Bit 31 is NOT an IRQ per se, it shows if connection to    --
	-- the BRK board is OK. Not resettable, read-only.           --
	---------------------------------------------------------------
	process_irq : process (nrst, clk, adc_read, adc_done_up, dac_written,
			dac_done_up, spi_broke, spi_connected, mach_irq_home,
			mach_irq_fault, mach_irq_done, mach_irq_cycle,
			mach_irq_probe, mach_irq_fovf)
		variable irq_set	: std_logic_vector(17 downto 0);
		variable irq_clr	: std_logic_vector(17 downto 0);
		variable irq_val	: std_logic_vector(17 downto 0);
	begin
		irq_set := mach_irq_home & mach_irq_probe & mach_irq_fovf &
			mach_irq_fault & mach_irq_done & mach_irq_cycle &
			spi_broke & adc_done_up & dac_done_up;
		irq_clr := "000000000000" & adc_read & dac_written;
		if (nrst='0') then
			irqstat_reg <= (others => '0');
			irq_val := (others => '0');
		elsif (rising_edge(clk)) then
			mach_fovf_clr <= '0';
			if (irqstat_wr='1') then
				irq_clr := irq_clr or lb_din(17 downto 0);
				mach_fovf_clr <= lb_din(10);
			end if;
			irq_val := irq_val or irq_set;
			irq_val := irq_val and not irq_clr;
		end if;
		irqstat_reg <= (others => '0');
		irqstat_reg(17 downto 0) <= irq_val;
		irqstat_reg(31) <= spi_connected;
	end process;


	-- PCI buffers are always enabled when CPLD is up and running
	pci_en <= '0';

	-- Only one CS per SPI is currently used.
	lss12 <= '0';
	lss13 <= '0';
	lss22 <= '0';
	lss23 <= '0';

	ioaddr <= lb_adr(8 downto 2);

	irqstat_wr <= '1' when (ioaddr=IRQSTAT_REG_ADR and lb_wr='1') else '0';

	mach_enc_rd <= '1' when (ioaddr=MACH3ENC_REG_ADR and lb_rd='1') else '0';

	mach_enc_data <= d_in_reg(31 downto 24);

	nrst <= l_rst;
	clk <= l_clk;

	irq <= '1' when ((irqstat_reg and irqmask_reg)=x"0000_0000") else '0';
	lb_nint <= irq;

	d_out(31 downto 12) <= d_out_reg(31 downto 12);

	axes_map: for i in 0 to 5 generate
		d_out(2*i+1 downto 2*i) <= d_out_reg(2*i+1 downto 2*i) when (axes_active(i)='0') else pulser_bus(2*i+1 downto 2*i);
	end generate;
end guts;
