--------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Extension Bus GPIO implementation. --
--                                                                --
-- This is not the final intended use for the Extension Bus. It   --
-- can be used for multiple purposes (i.e. one of such planned    --
-- uses is multi Quadrature Encoders Interface.) This particular  --
-- code makes that bus somewhat useful for building extension     --
-- boards off of it. It provides parallel GPIO functionality for  --
-- Extension Bus.                                                 --
--                                                                --
-- Functionality is very simple -- there are Data and Direction   --
-- registers for each part of the bus (32 bit lower part and 10   --
-- bit higher part.) Direction is '1' for output, '0' for input.  --
-- Output data is latched in output register on register write,   --
-- inputs are latched on each clk that happens to be PCI clock.   --
-- No interrupts, no debouncing, plain and simple.                --
--------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity extbus_gpio is
generic (
	dreg_low_adr	: std_logic_vector(6 downto 0) := b"001_0111";
	dreg_hi_adr	: std_logic_vector(6 downto 0) := b"001_1000";
	ddir_low_adr	: std_logic_vector(6 downto 0) := b"001_1001";
	ddir_hi_adr	: std_logic_vector(6 downto 0) := b"001_1010"
);
port (
	ext_bus		: inout std_logic_vector(41 downto 0);
	data_in		: in std_logic_vector(31 downto 0);
	data_out	: out std_logic_vector(41 downto 0);
	dir_out		: out std_logic_vector(41 downto 0);
	ioaddr		: in std_logic_vector(6 downto 0);
	rd		: in std_logic;
	wr		: in std_logic;
	clk		: in std_logic;
	nrst		: in std_logic
);
end extbus_gpio;

architecture guts of extbus_gpio is
	-------------
	-- Signals --
	-------------
	signal data_reg	: std_logic_vector(41 downto 0);
	signal dir_reg	: std_logic_vector(41 downto 0);
begin
	data_latch : process (nrst, clk)
	begin
		if (nrst='0') then
			data_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			for i in 0 to 31 loop
				if (dir_reg(i)='0') then
					data_reg(i) <= ext_bus(i);
				elsif (ioaddr=dreg_low_adr and wr='1') then
					data_reg(i) <= data_in(i);
				end if;
			end loop;
			for i in 32 to 41 loop
				if (dir_reg(i)='0') then
					data_reg(i) <= ext_bus(i);
				elsif (ioaddr=dreg_hi_adr and wr='1') then
					data_reg(i) <= data_in(i-32);
				end if;
			end loop;
		end if;
	end process;


	dir_write : process (nrst, clk)
	begin
		if (nrst='0') then
			dir_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			if (ioaddr=ddir_low_adr and wr='1') then
				dir_reg(31 downto 0) <= data_in;
			elsif (ioaddr=ddir_hi_adr and wr='1') then
				dir_reg(41 downto 32) <= data_in(9 downto 0);
			end if;
		end if;
	end process;


	ext_pins : for i in ext_bus'range generate
		ext_bus(i) <= data_reg(i) when (dir_reg(i)='1') else 'Z';
	end generate;


	data_out <= data_reg;
	dir_out <= dir_reg;
end guts;
