-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 specific stuff. 4 Quadrature Encoders.         --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;


entity mach3_encoders is
port (
	enc_rd		: in std_logic;
	enc_data	: in std_logic_vector(7 downto 0);
	enc_counts	: out std_logic_vector(31 downto 0);
	clk		: in std_logic;
	rst		: in std_logic
);
end mach3_encoders;

architecture guts of mach3_encoders is

	-------------
	-- Signals --
	-------------

       	type enc_cntrs is array (3 downto 0) of integer range -128 to 127;
       	signal cntrs		: enc_cntrs;
	signal enc_mid		: std_logic_vector(7 downto 0);
	signal enc_tail		: std_logic_vector(7 downto 0);
	signal cnt_dir		: std_logic_vector(3 downto 0);
	signal cnt_ena		: std_logic_vector(3 downto 0);
	signal cnt_dir_hold	: std_logic_vector(3 downto 0);
	signal cnt_ena_hold	: std_logic_vector(3 downto 0);

begin

	mid_latches_proc : process (clk)
	begin
		if (rising_edge(clk)) then
			enc_mid <= enc_data;
		end if;
	end process;

	tail_latches_proc : process (clk)
	begin
		if (rising_edge(clk)) then
			enc_tail <= enc_mid;
		end if;
	end process;

	mach3_cntrs_proc : process (rst, clk)
	begin
		if (rst='1') then
			for i in 0 to 3 loop
				cntrs(i) <= 0;
				cnt_ena_hold(i) <= '0';
				cnt_dir_hold(i) <= '0';
			end loop;
		elsif (rising_edge(clk)) then
			for i in 0 to 3 loop
				if (enc_rd='1') then
					cnt_ena_hold(i) <= cnt_ena(i);
					cnt_dir_hold(i) <= cnt_dir(i);
					cntrs(i) <= 0;
				elsif (cnt_ena_hold(i)='1') then
					if (cnt_dir_hold(i)='1') then
						cntrs(i) <= cntrs(i) + 1;
					else
						cntrs(i) <= cntrs(i) - 1;
					end if;
					cnt_ena_hold(i) <= '0';
				elsif (cnt_ena(i)='1') then
					if (cnt_dir(i)='1') then
						cntrs(i) <= cntrs(i) + 1;
					else
						cntrs(i) <= cntrs(i) - 1;
					end if;
				end if;
			end loop;
		end if;
	end process;

	cnt_controls : for i in 0 to 3 generate
		cnt_ena(i) <= enc_mid(i*2) xor enc_mid(i*2+1) xor enc_tail(i*2) xor enc_tail(i*2+1);
		cnt_dir(i) <= enc_mid(i*2) xor enc_tail(i*2+1);
	end generate;

	counts_out : for i in 0 to 3 generate
		enc_counts(i*8+7 downto i*8) <= std_logic_vector(to_signed(cntrs(i),8));
	end generate;
end guts;
