-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 specific stuff. This is main FSM.              --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;


entity mach3_fsm is
port (
	cycle_period	: in std_logic_vector(5 downto 0);
	step_start	: in std_logic_vector(3 downto 0);
	step_stop	: in std_logic_vector(4 downto 0);

	fifo_pop	: out std_logic;
	fifo_empty	: in std_logic;

	tmr_req		: out std_logic;
	fsm_busy	: out std_logic;
	
	step_on		: out std_logic;
	step_stb	: out std_logic;

	fsm_run		: in std_logic;

	clk		: in std_logic;
	rst		: in std_logic
);
end mach3_fsm;

architecture guts of mach3_fsm is

	-------------
	-- Signals --
	-------------

	signal timebase		: unsigned(5 downto 0);
	signal tick		: std_logic;

	signal tmr_cntr		: unsigned(2 downto 0);

	signal cy_cntr		: unsigned(5 downto 0);
	signal cy_beg		: std_logic;
	signal cy_end		: std_logic;

	signal step_beg		: std_logic;
	signal step_end		: std_logic;

	signal fifo_prev	: std_logic;
	signal fifo_rst		: std_logic;

	signal busy		: std_logic;

	signal step_beg_prev	: std_logic;

begin

	fifo_rst_proc : process (rst, fsm_run, clk)
	begin
		if (rst='1' or fsm_run='0') then
			fifo_rst <= '0';
			fifo_prev <= '1';
		elsif (rising_edge(clk)) then
			if (fifo_prev='1' and fifo_empty='0') then
				fifo_rst <= '1';
			else
				fifo_rst <= '0';
			end if;
			fifo_prev <= fifo_empty;
		end if;
	end process;


	timebase_proc : process (rst, fsm_run, fifo_rst, clk)
	begin
		if (rst='1' or fsm_run='0' or fifo_rst='1') then
			timebase <= (0 => '1', others => '0');
		elsif (rising_edge(clk)) then
			if (tick='1') then
				timebase <= (others => '0');
			else
				timebase <= timebase + 1;
			end if;
		end if;
	end process;


	step_proc : process (rst, fsm_run, clk)
	begin
		if (rst='1' or fsm_run='0') then
			step_on <= '0';
		elsif (rising_edge(clk)) then
			if (step_beg='1') then
				step_on <= '1';
			elsif (step_end='1') then
				step_on <= '0';
			end if;
		end if;
	end process;


	cycle_proc : process (rst, fsm_run, fifo_rst, clk)
	begin
		if (rst='1' or fsm_run='0' or fifo_rst='1') then
			cy_cntr <= (others => '0');
		elsif (rising_edge(clk)) then
			if (tick='1') then
				if (cy_end='1') then
					cy_cntr <= (others => '0');
				else
					cy_cntr <= cy_cntr + 1;
				end if;
			end if;
		end if;
	end process;


	tmr_proc : process (rst, fsm_run, fifo_rst, clk)
	begin
		if (rst='1' or fsm_run='0' or fifo_rst='1') then
			tmr_cntr <= (others => '0');
		elsif (rising_edge(clk)) then
			if (cy_end='1' and tick='1') then
				if (tmr_cntr(2)='1') then
					tmr_cntr <= (others => '0');
				else
					tmr_cntr <= tmr_cntr + 1;
				end if;
			end if;
		end if;
	end process;


	fsm_busy_proc : process (rst, fsm_run, clk)
	begin
		if (rst='1' or fsm_run='0') then
			busy <= '0';
		elsif (rising_edge(clk)) then
			if (fifo_empty='0') then
				busy <= '1';
			elsif (fifo_empty='1' and cy_end='1') then
				busy <= '0';
			end if;
		end if;
	end process;


	step_stb_proc : process (rst, fsm_run, clk)
	begin
		if (rst='1' or fsm_run='0') then
			step_stb <= '0';
			step_beg_prev <= '0';
		elsif (rising_edge(clk)) then
			if (step_beg_prev='0' and step_beg='1') then
				step_stb <= '1';
			else
				step_stb <= '0';
			end if;
			step_beg_prev <= step_beg;
		end if;
	end process;
	

	tick <= timebase(5);

	step_beg <= '1' when (std_logic_vector(cy_cntr)=("00" & step_start)) else '0';
	step_end <= '1' when (std_logic_vector(cy_cntr)=('0' & step_stop)) else '0';

	cy_beg <= '1' when (cy_cntr="000000" and timebase="000000") else '0';
	cy_end <= '1' when (std_logic_vector(cy_cntr)=cycle_period) else '0';

	fifo_pop <= '1' when (cy_end='1' and tick='1' and fifo_empty='0') else '0';

	tmr_req <= '1' when (cy_beg='1' and tmr_cntr="000") else '0';

	fsm_busy <= busy;
end guts;
