-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 specific stuff.                                --
--                                                               --
-- This is the main part, the glue that connects all the bits    --
-- and pieces together and then it is included into the top      --
-- level file, centipede_pci, as a component.                    --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;
use work.mach3_bits.all;


entity mach3_main is
generic (
	fifo_depth	: natural := 20;
	idx_adr		: index_adr := (b"000_1011", b"000_1100", b"000_1101",
					b"000_1110", b"000_1111", b"001_0000");
	cfg_adr		: std_logic_vector(6 downto 0) := b"001_0001";
	cfg2_adr	: std_logic_vector(6 downto 0) := b"001_0010";
	ctl_adr		: std_logic_vector(6 downto 0) := b"001_0011";
	stat_adr	: std_logic_vector(6 downto 0) := b"001_0100";
	fifo_adr	: std_logic_vector(6 downto 0) := b"001_0101"
);
port (
	pulser_bus	: out std_logic_vector(11 downto 0);
	axes_active	: out std_logic_vector(5 downto 0);
	irq_home	: out std_logic_vector(5 downto 0);
	irq_probe	: out std_logic;
	irq_cycle	: out std_logic;
	irq_done	: out std_logic;
	irq_fault	: out std_logic;
	irq_fovf	: out std_logic;
	enc_rd		: in std_logic;
	idx		: out index_regs;
	cfg_reg		: out std_logic_vector(31 downto 0);
	cfg2_reg	: out std_logic_vector(7 downto 0);
	ctl_reg		: out std_logic_vector(31 downto 0);
	stat_reg	: out std_logic_vector(31 downto 0);
	enc_reg		: out std_logic_vector(31 downto 0);
	fovf_clr	: in std_logic;
	enc_data	: in std_logic_vector(7 downto 0);
	data_in		: in std_logic_vector(31 downto 0);
	din_reg		: in std_logic_vector(31 downto 0);
	ioaddr		: in std_logic_vector(6 downto 0);
	rd		: in std_logic;
	wr		: in std_logic;
	clk		: in std_logic;
	nrst		: in std_logic
);
end mach3_main;

architecture guts of mach3_main is

	-------------
	-- Signals --
	-------------

	signal mach3_cfg_reg	: std_logic_vector(31 downto 0);
	signal mach3_cfg2_reg	: std_logic_vector(7 downto 0);
	signal mach3_ctl_reg	: std_logic_vector(31 downto 0);
	signal mach3_stat_reg	: std_logic_vector(31 downto 0);

	signal idx_write	: std_logic_vector(5 downto 0);
	signal cfg_write	: std_logic;
	signal cfg2_write	: std_logic;
	signal ctl_write	: std_logic;
	signal stat_write	: std_logic;

	signal reset		: std_logic;	-- Active high

	signal idx_regs		: index_regs;

	signal axis_en		: std_logic_vector(5 downto 0);

	signal lim_pos_in	: std_logic_vector(5 downto 0);
	signal lim_pos_hit	: std_logic_vector(5 downto 0);
	signal lim_pos_clr	: std_logic_vector(5 downto 0);
	signal lim_neg_in	: std_logic_vector(5 downto 0);
	signal lim_neg_hit	: std_logic_vector(5 downto 0);
	signal lim_neg_clr	: std_logic_vector(5 downto 0);
	signal home_en		: std_logic_vector(5 downto 0);
	signal home_sw_in	: std_logic_vector(5 downto 0);
	signal home_hit		: std_logic_vector(5 downto 0);
	signal home_clr		: std_logic_vector(5 downto 0);

	signal dirs_out		: std_logic_vector(5 downto 0);
	signal dirs_inv		: std_logic_vector(5 downto 0);
	signal steps_out	: std_logic_vector(5 downto 0);
	signal steps_inv	: std_logic_vector(5 downto 0);

	signal fifo_push	: std_logic;
	signal fifo_pop		: std_logic;
	signal fifo_purge	: std_logic;
	signal step_on		: std_logic;
	signal step_stb_raw	: std_logic;
	signal step_stb_gated	: std_logic;
	signal fifo_out		: std_logic_vector(11 downto 0);
	signal fifo_in		: std_logic_vector(11 downto 0);
	signal probe_hit	: std_logic;
	signal probe_clr	: std_logic;

	signal st_start		: std_logic_vector(3 downto 0);
	signal st_stop		: std_logic_vector(4 downto 0);
	signal cy_stop		: std_logic_vector(5 downto 0);

	signal no_limits	: std_logic;

	signal simulate		: std_logic;

	signal probe_in		: std_logic;
	signal rst_in		: std_logic;

	signal estop_switch	: std_logic;
	signal estop_sw_in	: std_logic;
	signal estop_req_in	: std_logic;
	signal estop_lim_in	: std_logic;
	signal estop_clr	: std_logic;

	signal f_pop		: std_logic;
	
	signal stat_estop	: std_logic;
	signal stat_fault	: std_logic;

	signal f_room		: std_logic_vector(4 downto 0);
	signal f_full		: std_logic;
	signal f_empty		: std_logic;
	signal fsm_busy		: std_logic;
	signal fsm_busy_prev	: std_logic;

	signal req_cycle	: std_logic;
	signal req_done		: std_logic;
	signal req_fovf		: std_logic;

	signal f_ovf_clr	: std_logic;

	signal mach3_run	: std_logic;

begin

	--=====================--
	-- COMPONENT INSTANCES --
	--=====================--

	----------------------------
	-- Motion (Step/Dir) FIFO --
	----------------------------

	z1: component mach3_fifo
	generic map (
		fifo_depth => fifo_depth
	)
	port map (
		fifo_in => fifo_in,
		fifo_out => fifo_out,
		fifo_push => fifo_push,
		fifo_pop => fifo_pop,
		fifo_purge => fifo_purge,
		fifo_full => f_full,
		fifo_empty => f_empty,
		fifo_ovf_irq => req_fovf,
		fifo_ovf_clr => f_ovf_clr,
		fifo_room => f_room,
		clk => clk,
		rst => reset
	);

	---------------------------------------------
	-- Position (Index) registers and latches. --
	---------------------------------------------

	z2: component mach3_index
	port map (
		data_in => data_in,
		steps => steps_out,
		step_stb => step_stb_gated,
		dirs => dirs_out,
		idx_regs => idx_regs,
		home_sw_hit => home_hit,
		probe_hit => probe_hit,
		idx_wr => idx_write,
		clk => clk,
		rst => reset
	);

	--------------------------------------------
	-- Home/Probe/Limit switches for all axes --
	--------------------------------------------

	z3: component mach3_switches
	port map (
		sw_probe => probe_in,
		sw_home => home_sw_in,
		sw_lim_neg => lim_neg_in,
		sw_lim_pos => lim_pos_in,
		lim_neg_hit => lim_neg_hit,
		lim_pos_hit => lim_pos_hit,
		probe_hit => probe_hit,
		home_hit => home_hit,
		lim_neg_clr => lim_neg_clr,
		lim_pos_clr => lim_pos_clr,
		probe_clr => probe_clr,
		home_clr => home_clr,
		clk => clk,
		rst => reset
	);

	---------------------
	-- Sequencer (FSM) --
	---------------------

	z4: component mach3_fsm
	port map (
		cycle_period => cy_stop,
		step_start => st_start,
		step_stop => st_stop,
		fifo_pop => f_pop,
		fifo_empty => f_empty,
		tmr_req => req_cycle,
		fsm_busy => fsm_busy,
		step_on => step_on,
		step_stb => step_stb_raw,
		fsm_run => mach3_run,
		clk => clk,
		rst => reset
	);

	--------------
	-- Encoders --
	--------------

	z5: component mach3_encoders
	port map (
		enc_rd => enc_rd,
		enc_data => enc_data,
		enc_counts => enc_reg,
		clk => clk,
		rst => reset
	);

	-----------------------------
	-- Hardware/software reset --
	-----------------------------
	reset_proc : process (nrst, clk)
	begin
		if (nrst='0') then
			reset <= '1';
		elsif (rising_edge(clk)) then
			if (rst_in='1') then
				reset <= '1';
			else
				reset <= '0';
			end if;
		end if;
	end process;


	----------------------------------
	-- Configuration register write --
	----------------------------------
	cfg_write_proc : process (reset, clk)
	begin
		if (reset='1') then
			mach3_cfg_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			if (cfg_write='1') then
				mach3_cfg_reg <= data_in;
			end if;
		end if;
	end process;


	-----------------------------------------
	-- Second configuration register write --
	-----------------------------------------
	cfg2_write_proc : process (reset, clk)
	begin
		if (reset='1') then
			mach3_cfg2_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			if (cfg2_write='1') then
				mach3_cfg2_reg <= data_in(7 downto 0);
			end if;
		end if;
	end process;


	----------------------------
	-- Control register write --
	----------------------------
	ctl_write_proc : process (reset, clk)
	begin
		if (reset='1') then
			mach3_ctl_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			if (ctl_write='1') then
				mach3_ctl_reg(31 downto 16) <= data_in(31 downto 16);
				mach3_ctl_reg(15) <= '0';
				mach3_ctl_reg(14 downto 0) <= data_in(14 downto 0);
			end if;
		end if;
	end process;


	------------------------
	-- FSM Done Interrupt --
	------------------------
	req_delay : process (clk)
	begin
		if (rising_edge(clk)) then
			fsm_busy_prev <= fsm_busy;
		end if;
	end process;


	------------------
	-- EStop Switch --
	------------------
	estop_sw_proc : process (reset, clk)
	begin
		if (reset='1') then
			estop_sw_in <= '0';
		elsif (rising_edge(clk)) then
			if (estop_switch='0') then
				estop_sw_in <= '1';
			elsif (estop_clr='1') then
				estop_sw_in <= '0';
			end if;
		end if;
	end process;


	----------------------------------
	-- Concurrent code and mappings --
	----------------------------------

	fifo_push <= '1' when (ioaddr=fifo_adr and wr='1') else '0';
	fifo_purge <= ctl_write and data_in(15);
	fifo_in <= data_in(11 downto 0);
	fifo_pop <= f_pop and not stat_estop;

	idx_write_map : for i in 0 to 5 generate
		idx_write(i) <= '1' when (ioaddr=idx_adr(i) and wr='1') else '0';
	end generate;

	cfg_write <= '1' when (ioaddr=cfg_adr and wr='1') else '0';
	cfg2_write <= '1' when (ioaddr=cfg2_adr and wr='1') else '0';
	ctl_write <= '1' when (ioaddr=ctl_adr and wr='1') else '0';
	stat_write <= '1' when (ioaddr=stat_adr and wr='1') else '0';

	axis_mapping : for i in 0 to 5 generate
		axis_en(i) <= mach3_cfg_reg(i*5);
		dirs_inv(i) <= mach3_cfg_reg(i*5+1);
		steps_inv(i) <= mach3_cfg2_reg(i);
		home_en(i) <= mach3_cfg_reg(i*5+4);

		lim_pos_clr(i) <= data_in(i*3) and stat_write;
		lim_neg_clr(i) <= data_in(i*3+1) and stat_write;
		home_clr(i) <= data_in(i*3+2) and stat_write;

		mach3_stat_reg(i*3) <= lim_pos_hit(i);
		mach3_stat_reg(i*3+1) <= lim_neg_hit(i);
		mach3_stat_reg(i*3+2) <= home_hit(i);

		lim_pos_in(i) <= (din_reg(i*3) xnor mach3_cfg_reg(i*5+2))
			and axis_en(i);
		lim_neg_in(i) <= (din_reg(i*3+1) xnor mach3_cfg_reg(i*5+2))
			and axis_en(i);
		home_sw_in(i) <= (din_reg(i*3+2) xnor mach3_cfg_reg(i*5+3))
			and axis_en(i) and home_en(i);

		dirs_out(i) <= fifo_out(i*2);
		steps_out(i) <= fifo_out(i*2+1);
--		steps_out(i) <= fifo_out(i*2+1) and axis_en(i);
	end generate;

	probe_in <= (din_reg(18) xnor mach3_cfg_reg(30)) and mach3_cfg_reg(31);
	probe_clr <= data_in(18) and stat_write;

	st_start <= mach3_ctl_reg(9 downto 6);
	st_stop <= mach3_ctl_reg(14 downto 10);
	cy_stop <= mach3_ctl_reg(5 downto 0);

	rst_in <= mach3_ctl_reg(31);

	mach3_stat_reg(18) <= probe_hit;
	mach3_stat_reg(23 downto 19) <= f_room;
	mach3_stat_reg(24) <= f_empty;
	mach3_stat_reg(25) <= f_full;

	mach3_stat_reg(28 downto 26) <= "000";

	mach3_stat_reg(29) <= fsm_busy;

	no_limits <= mach3_ctl_reg(29);
	simulate <= mach3_ctl_reg(27);

	estop_switch <= din_reg(19);

	estop_req_in <= mach3_ctl_reg(28);
	estop_lim_in <= (lim_pos_hit(0) or lim_neg_hit(0) or
		lim_pos_hit(1) or lim_neg_hit(1) or
		lim_pos_hit(2) or lim_neg_hit(2) or
		lim_pos_hit(3) or lim_neg_hit(3) or
		lim_pos_hit(4) or lim_neg_hit(4) or
		lim_pos_hit(5) or lim_neg_hit(5)) and not no_limits;
	estop_clr <= data_in(30) and stat_write;

	pulser : for i in 0 to 5 generate
		pulser_bus(i*2) <= dirs_out(i) xor dirs_inv(i);
		pulser_bus(i*2+1) <= ((steps_out(i) and step_on) xor steps_inv(i))
			when (stat_estop='0' and simulate='0') else steps_inv(i);
	end generate;

	stat_fault <= estop_sw_in or estop_lim_in;
	stat_estop <= stat_fault or estop_req_in;

	mach3_stat_reg(30) <= stat_estop;

	cfg_reg <= mach3_cfg_reg;
	cfg2_reg <= mach3_cfg2_reg;
	ctl_reg <= mach3_ctl_reg;
	stat_reg <= mach3_stat_reg;

	axes_active <= axis_en;
	idx <= idx_regs;

	f_ovf_clr <= fovf_clr;

	mach3_run <= mach3_ctl_reg(30);
	mach3_stat_reg(31) <= mach3_run;

	step_stb_gated <= step_stb_raw and fsm_busy;

	----------
	-- IRQs --
	----------
	req_done <= fsm_busy_prev and not fsm_busy;

	irq_home <= home_hit;
	irq_probe <= probe_hit;
	irq_cycle <= req_cycle;
	irq_done <= req_done;
	irq_fovf <= req_fovf;

	irq_fault <= stat_fault;
end guts;
